/*  Skeleton Program code for the AQA AS1 Summer 2020 examination
    this code should be used in conjunction with the Preliminary Material
    written by the AQA AS1 Programmer Team
    developed in the NetBeans IDE 8.1 environment

    Version number: 0.0.0

*/
package aqa.graphics;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

public class AqaGraphics {

    static final String EMPTY_STRING = "";
    static final int MAX_WIDTH = 100;
    static final int MAX_HEIGHT = 100;

    class FileHeader { 
        public String title = EMPTY_STRING;
        public int width = MAX_WIDTH;
        public int height = MAX_HEIGHT;
        public String fileType = EMPTY_STRING;
    }

    void displayError(String errorMessage) {
        Console.writeLine("Error: " + errorMessage);
    }

    void printHeading(String heading) {
        Console.writeLine(heading);
        int headingLength = heading.length();
        for (int position = 1; position <= headingLength; position++) {
            Console.write("=");      
        }        
        Console.writeLine();
    }

    void displayImage(String[][] grid, FileHeader header) {
        Console.writeLine();
        printHeading(header.title);
        for (int thisRow = 0; thisRow < header.height; thisRow++) {
            for (int thisColumn = 0; thisColumn < header.width; thisColumn++) {
                Console.write(grid[thisRow][thisColumn]);
            }
            Console.writeLine();
        }
    }

    void saveImage(String[][] grid, FileHeader header) {
        String fileName;
        BufferedWriter fileOut;
        Console.writeLine("The current title of your image is: " + header.title);
        Console.write("Do you want to use this as your filename? (Y/N) ");
        String answer = Console.readLine();
        if (answer.equals("N") || answer.equals("n")) {
            Console.write("Enter a new filename: ");
            fileName = Console.readLine();
        } else {
            fileName = header.title;
        }
        try {
            fileOut = new BufferedWriter(new FileWriter(fileName + ".txt", false));
            fileOut.write(header.title + "\n");
            for (int row = 0; row < header.height; row++) {
                for (int column = 0; column < header.width; column++) {
                    fileOut.write(grid[row][column]);
                }
                fileOut.write("\n");
            }
            fileOut.close();
        } catch (IOException e) {
        }
    }

    void editImage(String[][] grid, FileHeader header) {
        displayImage(grid, header);
        String answer = EMPTY_STRING;
        while (!answer.equals("N")) {
            String symbol = EMPTY_STRING;
            String newSymbol = EMPTY_STRING;
            while (symbol.length() != 1) {
                Console.write("Enter the symbol you want to replace: ");
                symbol = Console.readLine();
            }
            while (newSymbol.length() != 1) {
                Console.write("Enter the new symbol: ");
                newSymbol = Console.readLine();
            }
            for (int thisRow = 0; thisRow < header.height; thisRow++) {
                for (int thisColumn = 0; thisColumn < header.width; thisColumn++) {
                    if (grid[thisRow][thisColumn].equals(symbol)) {
                        grid[thisRow][thisColumn] = newSymbol;
                    }
                }
            }
            displayImage(grid, header);
            Console.write("Do you want to make any further changes? (Y/N) ");
            answer = Console.readLine();
        }
    }

    String convertChar(int pixelValue) {
        String asciiChar;
        if (pixelValue <= 32) {
            asciiChar = "#";
        } else if (pixelValue <= 64) {
            asciiChar = "&";
        } else if (pixelValue <= 96) {
            asciiChar = "+";
        } else if (pixelValue <= 128) {
            asciiChar = ";";
        } else if ( pixelValue <= 160) {
            asciiChar = ":";
        } else if ( pixelValue <= 192) {
            asciiChar = ",";
        } else if ( pixelValue <= 224) {
            asciiChar = ".";
        } else {
            asciiChar = " ";
        }
        return asciiChar;
    }

    void loadGreyScaleImage(BufferedReader fileIn, String[][] grid, FileHeader header) {
        try {
            for (int row = 0; row < header.height; row++) {
                for (int column = 0; column < header.width; column++) {
                    String nextPixel = fileIn.readLine();
                    int pixelValue = Integer.parseInt(nextPixel);
                    grid[row][column] = convertChar(pixelValue);
                }
            }                  
        } catch (Exception e) {
            displayError("Image data error");
        }
    }

    void loadAsciiImage(BufferedReader fileIn, String[][] grid, FileHeader header) {
        String imageData;
        try {
            imageData = fileIn.readLine();
            int nextChar = 0;
            for (int row = 0; row < header.height; row++) {
                for (int column = 0; column < header.width; column++) {
                    grid[row][column] = imageData.charAt(nextChar)+"";
                    nextChar += 1;  
                }
            }
        } catch (Exception e) {
            displayError("Image data error");
        }
    }

    void loadFile(String[][] grid, FileHeader header) {
        boolean fileFound = false;
        boolean fileTypeOK = false;
        Console.write("Enter filename to load: ");
        String fileName = Console.readLine();
        try {
            BufferedReader fileIn = new BufferedReader(new FileReader(fileName + ".txt"));
            fileFound = true;
            String headerLine = fileIn.readLine();
            String[] fields = headerLine.split(",");
            header.title = fields[0];
            header.width = Integer.parseInt(fields[1]);
            header.height = Integer.parseInt(fields[2]);
            header.fileType = fields[3];
            header.fileType = header.fileType.charAt(0) + "";
            if (header.fileType.equals("A")) { 
                loadAsciiImage(fileIn, grid, header);
                fileTypeOK = true;
            } else if ( header.fileType.equals("G")){
                loadGreyScaleImage(fileIn, grid, header);
                fileTypeOK = true;
            }
            fileIn.close();
            if (!fileTypeOK) {
                displayError("Unknown file type");
            } else {
                displayImage(grid, header);
            }
        } catch (IOException e) {
            if (!fileFound) {
                displayError("File not found");
            } else {
                displayError("Unknown error");
            }
        }
    }

    void saveFile(String[][] grid, FileHeader header) {
        Console.write("Enter filename: ");
        String fileName = Console.readLine();
        try {
            BufferedWriter fileOut = new BufferedWriter(new FileWriter(fileName + ".txt"));
            fileOut.write(header.title + "," + header.width + "," + header.height + "," + "A" + "\n");
            for (int row = 0; row < header.height; row++) {
                for (int column = 0; column < header.width; column++) {
                    fileOut.write(grid[row][column]);
                }
            }
            fileOut.close();            
        } catch (IOException e) {
        }
    }                

    void clearGrid(String[][] grid) {
        for (int row = 0; row < MAX_HEIGHT; row++) {
            for (int column = 0; column < MAX_WIDTH; column++) {
                grid[row][column] = ".";
            }
        }
    }

    void displayMenu() {
        Console.writeLine();
        Console.writeLine("Main Menu");
        Console.writeLine("=========");
        Console.writeLine("L - Load graphics file"); 
        Console.writeLine("D - Display image");
        Console.writeLine("E - Edit image");
        Console.writeLine("S - Save image");
        Console.writeLine("X - Exit program"); 
        Console.writeLine();
    }

    char GetMenuOption() {
        String menuOption = EMPTY_STRING;
        while (menuOption.length() != 1) {
            Console.write("Enter your choice: ");
            menuOption = Console.readLine();
        }
        return menuOption.charAt(0);
    }

    void graphics() {
        String[][] grid = new String[MAX_HEIGHT][MAX_WIDTH];
        clearGrid(grid);
        FileHeader header = new FileHeader();
        boolean programEnd = false;
        while (!programEnd) {
            displayMenu();
            char menuOption = GetMenuOption();
            if ( menuOption == 'L') {
                loadFile(grid, header);
            } else if ( menuOption == 'D') {
                displayImage(grid, header);
            } else if ( menuOption == 'E') {
                editImage(grid, header); 
            } else if ( menuOption == 'S') {    
                saveImage(grid, header);
            } else if ( menuOption == 'X') {
                programEnd = true;
            } else {
                Console.writeLine("You did not choose a valid menu option. Try again");
            }
        }
        Console.writeLine("You have chosen to exit the program");
        Console.write("Do you want to save the image as a graphics file? (Y/N) ");
        String answer = Console.readLine();
        if ( answer.equals("Y") || answer.equals("y")) {
            saveFile(grid, header);
        }
    }
    
    public static void main(String[] args) {
        AqaGraphics graphics = new AqaGraphics();
        graphics.graphics();
    }
}
