program project1;

// Skeleton Program for the AQA AS1 Summer 2020 examination
// this code should be used in conjunction with the Preliminary Material
// written by the AQA AS1 Programmer Team
// Developed in a Delphi programming environment (Lazarus IDE v1.8.4) Free Pascal

// NOTE debugger should be switched off

// Version number: 0.0.0

{$APPTYPE CONSOLE}

uses SysUtils, StrUtils;

const
  EMPTY_STRING = '';
  MAX_WIDTH = 100;
  MAX_HEIGHT = 100;

type
  FileHeader = record
    Title: string;
    Width, Height: integer;
    FileType: string;
  end;

  TGrid = array[0 .. MAX_HEIGHT - 1, 0 .. MAX_WIDTH - 1] of string;

procedure SetHeader(var Header: FileHeader);
begin
  Header.Title := EMPTY_STRING;
  Header.Width := MAX_WIDTH;
  Header.Height := MAX_HEIGHT;
  Header.FileType := EMPTY_STRING;
end;

procedure DisplayError(ErrorMessage: string);
begin
  writeln('Error: ', ErrorMessage);
end;

procedure PrintHeading(Heading: string);
var
  HeadingLength, Position: integer;
begin
  writeln(Heading);
  HeadingLength := length(Heading);
  for Position := 1 to HeadingLength do
    write('=');
  writeln;
end;

procedure DisplayImage(Grid: TGrid; Header: FileHeader);
var
  ThisRow, ThisColumn: integer;
begin
  writeln;
  PrintHeading(Header.Title);
  for ThisRow := 0 to Header.Height - 1 do
    begin
      for ThisColumn := 0 to Header.Width - 1 do
        write(Grid[ThisRow, ThisColumn]);
      writeln;
    end;
end;

procedure SaveImage(Grid: TGrid; Header: FileHeader);
var
  FileName, Answer: string;
  FileOut: text;
  Row, Column: integer;
begin
  writeln('The current title of your image is: ', Header.Title);
  write('Do you want to use this as your filename? (Y/N) ');
  readln(Answer);
  if (Answer = 'N') or (Answer = 'n')
    then
      begin
        write('Enter a new filename: ');
        readln(FileName);
      end
    else
      FileName := Header.Title;
  assignFile(FileOut, FileName + '.txt');
  rewrite(FileOut);
  writeln(FileOut, Header.Title);
  for Row := 0 to Header.Height - 1 do
    begin
      for Column := 0 to Header.Width - 1 do
        write(FileOut, Grid[Row, Column]);
      writeln(FileOut);
    end;
  closeFile(FileOut);
end;

procedure EditImage(var Grid: TGrid; Header: FileHeader);
var
  Answer, Symbol, NewSymbol: string;
  ThisRow, ThisColumn: integer;

begin
  DisplayImage(Grid, Header);
  Answer := EMPTY_STRING;
  while Answer <> 'N' do
  begin
    Symbol := EMPTY_STRING;
    NewSymbol := EMPTY_STRING;
    while length(Symbol) <> 1 do
    begin
      write('Enter the symbol you want to replace: ');
      readln(Symbol);
    end;
    while length(NewSymbol) <> 1 do
    begin
      write('Enter the new symbol: ');
      readln(NewSymbol);
    end;
    for ThisRow := 0 to Header.Height - 1 do
      for ThisColumn := 0 to Header.Width - 1 do
        if Grid[ThisRow, ThisColumn] = Symbol
          then
            Grid[ThisRow, ThisColumn] := NewSymbol;
    DisplayImage(Grid, Header);
    write('Do you want to make any further changes? (Y/N) ');
    readln(Answer);
  end;
end;

function ConvertChar(PixelValue: integer): string;
var
  AsciiChar: string;
begin
  if PixelValue <= 32
    then
      AsciiChar := '#'
    else
      begin
        case PixelValue of
           33 ..  64: AsciiChar := '&';
           65 ..  96: AsciiChar := '+';
           97 .. 128: AsciiChar := ';';
          129 .. 160: AsciiChar := ':';
          161 .. 192: AsciiChar := ',';
          193 .. 224: AsciiChar := '.';
        else
          AsciiChar := ' ';
        end;
      end;
  ConvertChar := AsciiChar;
end;

procedure LoadGreyScaleImage(var FileIn: text; var Grid: TGrid; Header: FileHeader);
var
  Row, Column: integer;
  NextPixel: string;
  PixelValue: integer;
begin
  try
    for Row := 0 to Header.Height - 1 do
      for Column := 0 to Header.Width - 1 do
        begin
          readln(FileIn, NextPixel);
          PixelValue := strToInt(NextPixel);
          Grid[Row, Column] := ConvertChar(PixelValue);
        end;
  except
    DisplayError('Image data error');
  end;
end;

procedure LoadAsciiImage(var FileIn: text; var Grid: TGrid; Header: FileHeader);
var
  Row, Column: integer;
  NextPixel: char;
begin
    for Row := 0 to Header.Height - 1 do
      begin
        for Column := 0 to Header.Width - 1 do
          begin

            if EOF(FileIn)
              then
                begin
                  DisplayError('Image data error');
                  break;
                end;
            read(FileIn, NextPixel);
            Grid[Row, Column] := NextPixel;
          end;
        if EOF(FileIn)
          then
            begin
              DisplayError('Image data error');
              break;
            end;
      end;
end;

procedure LoadFile(var Grid: TGrid; var Header: FileHeader);
var
  FileFound, FileTypeOK: boolean;
  FileName, HeaderLine: string;
  FileIn: text;
  Fields: array[0 .. 3] of string;
  CommaPos, i : integer;
begin
  FileFound := false;
  FileTypeOK := false;
  write('Enter filename to load: ');
  readln(FileName);
  try
    assignFile(FileIn, FileName + '.txt');
    reset(FileIn);
    FileFound := true;
    readln(FileIn, HeaderLine);
    for i := 0 to 2 do
    begin
      CommaPos := pos(',', HeaderLine);
      Fields[i] := leftStr(HeaderLine, CommaPos - 1);
      HeaderLine := rightStr(Headerline, length(HeaderLine) - CommaPos);
    end;
    Fields[3] := HeaderLine;
    Header.Title := Fields[0];
    Header.Width := StrToInt(Fields[1]);
    Header.Height := StrToInt(Fields[2]);
    Header.FileType := Fields[3];
    if Header.FileType = 'A'
      then
        begin
          LoadAsciiImage(FileIn, Grid, Header);
          FileTypeOK := true;
        end
      else
        if Header.FileType = 'G'
          then
            begin
            LoadGreyScaleImage(FileIn, Grid, Header);
            FileTypeOK := true;
            end;
    closeFile(FileIn);
    if not FileTypeOK
      then
        DisplayError('Unknown file type')
      else
        DisplayImage(Grid, Header);
  except
    if not FileFound
      then
        DisplayError('File not found')
      else
        DisplayError('Unknown error');
  end;
end;

procedure SaveFile(Grid: TGrid; Header: FileHeader);
var
  FileName: string;
  FileOut: text;
  Row, Column: integer;
begin
  write('Enter filename: ');
  readln(FileName);
  assignFile(FileOut, FileName + '.txt');
  rewrite(FileOut);
  writeln(FileOut, Header.Title, ',', intToStr(Header.Width), ',', intToStr(Header.Height), ',', 'A');
  for Row := 0 to Header.Height - 1 do
      for Column := 0 to Header.Width - 1 do
        write(FileOut, Grid[Row, Column]);
  closeFile(FileOut);
end;

procedure ClearGrid(var Grid: TGrid);
var
  Row, Column: integer;
begin
  for Row := 0 to MAX_HEIGHT - 1 do
    for Column := 0 to MAX_WIDTH - 1 do
      Grid[Row, Column] := '.';
end;

procedure DisplayMenu();
begin
  writeln;
  writeln('Main Menu');
  writeln('=========');
  writeln('L - Load graphics file');
  writeln('D - Display image');
  writeln('E - Edit image');
  writeln('S - Save image');
  writeln('X - Exit program');
  writeln;
end;

function GetMenuOption(): char;
var
  MenuOption: string;
begin
  MenuOption := EMPTY_STRING;
  while length(MenuOption) <> 1 do
  begin
    write('Enter your choice: ');
    readln(MenuOption);
  end;
  GetMenuOption := MenuOption[1];
end;

procedure Graphics();
var
  Grid: TGrid;
  Header: FileHeader;
  ProgramEnd: boolean;
  MenuOption: char;
  Answer: char;
begin
  ClearGrid(Grid);
  SetHeader(Header);
  ProgramEnd := false;
  while not ProgramEnd do
  begin
    DisplayMenu();
    MenuOption := GetMenuOption();
    case MenuOption of
      'L': LoadFile(Grid, Header);
      'D': DisplayImage(Grid, Header);
      'E': EditImage(Grid, Header);
      'S': SaveImage(Grid, Header);
      'X': ProgramEnd := true;
    else
      writeln('You did not choose a valid menu option. Try again');
    end;
  end;
  writeln('You have chosen to exit the program');
  write('Do you want to save the image as a graphics file? (Y/N) ');
  readln(Answer);
  if (Answer = 'Y') or (Answer = 'y')
    then
      SaveFile(Grid, Header);
  readln;
end;

begin
  Graphics();
end.







