Imports System.IO

Module Module1
  ' Skeleton Program for the AQA AS1 Summer 2020 examination
  ' this code should be used in conjunction with the Preliminary Material
  ' written by the AQA AS1 Programmer Team
  ' developed in Visual Studio Community Edition

  ' Version number: 0.0.0

  Const EMPTY_STRING As String = ""
  Const MAX_WIDTH As Integer = 100
  Const MAX_HEIGHT As Integer = 100

  Structure FileHeader
    Dim Title As String
    Dim Width As Integer
    Dim Height As Integer
    Dim FileType As String
  End Structure

  Sub SetHeader(ByRef Header As FileHeader)
    Header.Title = EMPTY_STRING
    Header.Width = MAX_WIDTH
    Header.Height = MAX_HEIGHT
    Header.FileType = EMPTY_STRING
  End Sub

  Sub DisplayError(ErrorMessage As String)
    Console.WriteLine("Error: " & ErrorMessage)
  End Sub

  Sub PrintHeading(Heading As String)
    Console.WriteLine(Heading)
    Dim HeadingLength As Integer = Heading.Length
    For Position = 1 To HeadingLength
      Console.Write("=")
    Next
    Console.WriteLine()
  End Sub

  Sub DisplayImage(Grid(,) As String, Header As FileHeader)
    Console.WriteLine()
    PrintHeading(Header.Title)
    For ThisRow = 0 To Header.Height - 1
      For ThisColumn = 0 To Header.Width - 1
        Console.Write(Grid(ThisRow, ThisColumn))
      Next
      Console.WriteLine()
    Next
  End Sub

  Sub SaveImage(Grid(,) As String, Header As FileHeader)
    Console.WriteLine("The current title of your image is: " + Header.Title)
    Console.Write("Do you want to use this as your filename? (Y/N) ")
    Dim Answer As String = Console.ReadLine()
    Dim FileName As String
    If Answer = "N" Or Answer = "n" Then
      Console.Write("Enter a new filename: ")
      FileName = Console.ReadLine()
    Else
      FileName = Header.Title
    End If
    Dim FileOut As StreamWriter = New StreamWriter(FileName & ".txt")
    FileOut.WriteLine(Header.Title)
    For Row = 0 To Header.Height - 1
      For Column = 0 To Header.Width - 1
        FileOut.Write(Grid(Row, Column))
      Next
      FileOut.WriteLine()
    Next
    FileOut.Close()
  End Sub

  Sub EditImage(ByRef Grid(,) As String, ByVal Header As FileHeader)
    DisplayImage(Grid, Header)
    Dim Answer As String = EMPTY_STRING
    Dim Symbol As String
    Dim NewSymbol As String
    While Answer <> "N"
      Symbol = EMPTY_STRING
      NewSymbol = EMPTY_STRING
      While Symbol.Length <> 1
        Console.Write("Enter the symbol you want to replace: ")
        Symbol = Console.ReadLine()
      End While
      While NewSymbol.Length <> 1
        Console.Write("Enter the new symbol: ")
        NewSymbol = Console.ReadLine()
      End While
      For ThisRow = 0 To Header.Height - 1
        For ThisColumn = 0 To Header.Width - 1
          If Grid(ThisRow, ThisColumn) = Symbol Then
            Grid(ThisRow, ThisColumn) = NewSymbol
          End If
        Next
      Next
      DisplayImage(Grid, Header)
      Console.Write("Do you want to make any further changes? (Y/N) ")
      Answer = Console.ReadLine()
    End While
  End Sub

  Function ConvertChar(PixelValue As Integer) As String
    Dim AsciiChar As Char
    If PixelValue <= 32 Then
      AsciiChar = "#"
    ElseIf PixelValue <= 64 Then
      AsciiChar = "&"
    ElseIf PixelValue <= 96 Then
      AsciiChar = "+"
    ElseIf PixelValue <= 128 Then
      AsciiChar = ";"
    ElseIf PixelValue <= 160 Then
      AsciiChar = ":"
    ElseIf PixelValue <= 192 Then
      AsciiChar = ","
    ElseIf PixelValue <= 224 Then
      AsciiChar = "."
    Else
      AsciiChar = " "
    End If
    Return AsciiChar
  End Function

  Sub LoadGreyScaleImage(ByVal FileIn As StreamReader, ByRef Grid(,) As String, ByVal Header As FileHeader)
    Try
      Dim NextPixel As String
      Dim PixelValue As Integer
      For Row = 0 To Header.Height - 1
        For Column = 0 To Header.Width - 1
          NextPixel = FileIn.ReadLine()
          PixelValue = CInt(NextPixel)
          Grid(Row, Column) = ConvertChar(PixelValue)
        Next
      Next
    Catch
      DisplayError("Image data error")
    End Try
  End Sub

  Sub LoadAsciiImage(ByVal FileIn As StreamReader, ByRef Grid(,) As String, ByVal Header As FileHeader)
    Try
      Dim ImageData As String = FileIn.ReadLine()
      Dim NextChar As Integer = 0
      For Row = 0 To Header.Height - 1
        For Column = 0 To Header.Width - 1
          Grid(Row, Column) = ImageData(NextChar)
          NextChar += 1
        Next
      Next
    Catch
      DisplayError("Image data error")
    End Try
  End Sub

  Sub LoadFile(ByRef Grid(,) As String, ByRef Header As FileHeader)
    Dim FileFound As Boolean = False
    Dim FileTypeOK As Boolean = False
    Console.Write("Enter filename to load: ")
    Dim FileName As String = Console.ReadLine()
    Try
      Dim FileIn As StreamReader = New StreamReader(FileName & ".txt")
      FileFound = True
      Dim HeaderLine As String = FileIn.ReadLine()
      Dim Fields() As String = HeaderLine.Split(",")
      Header.Title = Fields(0)
      Header.Width = CInt(Fields(1))
      Header.Height = CInt(Fields(2))
      Header.FileType = Fields(3)
      Header.FileType = Header.FileType(0)
      If Header.FileType = "A" Then
        LoadAsciiImage(FileIn, Grid, Header)
        FileTypeOK = True
      ElseIf Header.FileType = "G" Then
        LoadGreyScaleImage(FileIn, Grid, Header)
        FileTypeOK = True
      End If
      FileIn.Close()
      If Not FileTypeOK Then
        DisplayError("Unknown file type")
      Else
        DisplayImage(Grid, Header)
      End If
    Catch
      If Not FileFound Then
        DisplayError("File not found")
      Else
        DisplayError("Unknown error")
      End If
    End Try
  End Sub

  Sub SaveFile(Grid(,) As String, Header As FileHeader)
    Console.Write("Enter filename: ")
    Dim FileName As String = Console.ReadLine()
    Dim FileOut As StreamWriter = New StreamWriter(FileName + ".txt")
    FileOut.WriteLine(Header.Title + "," + CStr(Header.Width) + "," + CStr(Header.Height) + "," + "A")
    For Row = 0 To Header.Height - 1
      For Column = 0 To Header.Width - 1
        FileOut.Write(Grid(Row, Column))
      Next
    Next
    FileOut.Close()
  End Sub

  Sub ClearGrid(ByRef Grid(,) As String)
    For Row = 0 To MAX_HEIGHT - 1
      For Column = 0 To MAX_WIDTH - 1
        Grid(Row, Column) = "."
      Next
    Next
  End Sub

  Sub DisplayMenu()
    Console.WriteLine()
    Console.WriteLine("Main Menu")
    Console.WriteLine("=========")
    Console.WriteLine("L - Load graphics file")
    Console.WriteLine("D - Display image")
    Console.WriteLine("E - Edit image")
    Console.WriteLine("S - Save image")
    Console.WriteLine("X - Exit program")
    Console.WriteLine()
  End Sub

  Function GetMenuOption() As Char
    Dim MenuOption As String = EMPTY_STRING
    While Len(MenuOption) <> 1
      Console.Write("Enter your choice: ")
      MenuOption = Console.ReadLine()
    End While
    Return MenuOption(0)
  End Function

  Sub Graphics()
    Dim MenuOption As Char
    Dim Grid(MAX_WIDTH - 1, MAX_HEIGHT - 1) As String
    ClearGrid(Grid)
    Dim Header As FileHeader
    SetHeader(Header)
    Dim ProgramEnd As Boolean = False
    While Not ProgramEnd
      DisplayMenu()
      MenuOption = GetMenuOption()
      If MenuOption = "L" Then
        LoadFile(Grid, Header)
      ElseIf MenuOption = "D" Then
        DisplayImage(Grid, Header)
      ElseIf MenuOption = "E" Then
        EditImage(Grid, Header)
      ElseIf MenuOption = "S" Then
        SaveImage(Grid, Header)
      ElseIf MenuOption = "X" Then
        ProgramEnd = True
      Else
        Console.WriteLine("You did not choose a valid menu option. Try again")
      End If
    End While
    Console.WriteLine("You have chosen to exit the program")
    Console.Write("Do you want to save the image as a graphics file? (Y/N) ")
    Dim Answer As String = Console.ReadLine()
    If Answer = "Y" Or Answer = "y" Then
      SaveFile(Grid, Header)
    End If
  End Sub

  Sub Main()
    Graphics()
  End Sub
End Module
